from datetime import datetime
from typing import ClassVar, Literal
from uuid import uuid4

from pydantic import Field

from moxn_types.blocks.base import BaseContent, BlockType


class SignedURLContentModel(BaseContent):
    block_type: ClassVar[Literal[BlockType.SIGNED]] = BlockType.SIGNED
    file_path: str
    expiration: datetime | None = None
    ttl_seconds: int = 3600
    buffer_seconds: int = 300
    signed_url: str | None = None


class SignedURLImageContentModel(SignedURLContentModel):
    type: Literal["url"] = "url"  # Assuming MediaDataImageFormat.URL is "url"
    media_type: Literal["image/jpeg", "image/png", "image/gif", "image/webp"]


class SignedURLPDFContentModel(SignedURLContentModel):
    type: Literal["url"] = "url"  # Assuming MediaDataPDFFormat.URL is "url"
    media_type: Literal["application/pdf"] = "application/pdf"
    filename: str = Field(default_factory=lambda: f"file-{uuid4()}.pdf")



"""Type definitions for Anthropic provider-specific content blocks."""

from typing import TYPE_CHECKING, Sequence, TypedDict, Union

if TYPE_CHECKING:
    # -- Anthropic --
    from anthropic.types import (
        Base64ImageSourceParam as AnthropicBase64ImageSourceParam,
    )
    from anthropic.types import (
        Base64PDFSourceParam as AnthropicBase64PDFSourceParam,
    )
    from anthropic.types import (
        DocumentBlockParam as AnthropicDocumentBlockParam,
    )
    from anthropic.types import (
        ImageBlockParam as AnthropicImageBlockParam,
    )
    from anthropic.types import Message as AnthropicMessage
    from anthropic.types import (
        TextBlockParam as AnthropicTextBlockParam,
    )
    from anthropic.types import (
        ToolResultBlockParam as AnthropicToolResultBlockParam,
    )
    from anthropic.types import (
        ToolUseBlockParam as AnthropicToolUseBlockParam,
    )
    from anthropic.types import URLImageSourceParam as AnthropicURLImageSourceParam
    from anthropic.types import (
        URLPDFSourceParam as AnthropicURLPDFSourceParam,
    )
    from anthropic.types.cache_control_ephemeral_param import (
        CacheControlEphemeralParam as AnthropicCacheControlEphemeralParam,
    )
    from anthropic.types.content_block import ContentBlock as AnthropicContentBlock
    from anthropic.types.message_param import MessageParam as AnthropicMessageParam
    from anthropic.types.tool_result_block_param import Content as AnthropicContent
    from anthropic.types.tool_use_block import ToolUseBlock as AnthropicToolUseBlock
else:
    AnthropicBase64PDFSourceParam = dict
    AnthropicURLPDFSourceParam = dict
    AnthropicBase64ImageSourceParam = dict
    AnthropicURLImageSourceParam = dict
    AnthropicDocumentBlockParam = dict
    AnthropicImageBlockParam = dict
    AnthropicMessageParam = dict
    AnthropicTextBlockParam = dict
    AnthropicToolUseBlockParam = dict
    AnthropicToolResultBlockParam = dict
    AnthropicCacheControlEphemeralParam = dict
    AnthropicContent = dict
    try:
        from anthropic.types.content_block import ContentBlock as AnthropicContentBlock
        from anthropic.types.message import Message as AnthropicMessage
        from anthropic.types.tool_use_block import ToolUseBlock as AnthropicToolUseBlock
    except ImportError:
        pass

# Anthropic content block types
AnthropicContentBlockParam = Union[
    AnthropicTextBlockParam,
    AnthropicImageBlockParam,
    AnthropicDocumentBlockParam,
    AnthropicToolUseBlockParam,
    AnthropicToolResultBlockParam,
]

AnthropicSystemContentBlockParam = AnthropicTextBlockParam
AnthropicDocumentSourceBlock = (
    AnthropicBase64PDFSourceParam | AnthropicURLPDFSourceParam
)
AnthropicImageSourceBlockParam = (
    AnthropicBase64ImageSourceParam | AnthropicURLImageSourceParam
)


# Provider-specific block sequences (for grouping operations)
AnthropicContentBlockParamSequence = Sequence[Sequence[AnthropicContentBlockParam]]
AnthropicSystemContentBlockParamSequence = Sequence[
    Sequence[AnthropicSystemContentBlockParam]
]


class AnthropicMessagesParam(TypedDict, total=False):
    system: str | list[AnthropicSystemContentBlockParam]
    messages: list[AnthropicContentBlockParam]



from pydantic import TypeAdapter

from moxn_types.blocks.document import PDFContentFromSourceModel
from moxn_types.blocks.image import ImageContentFromSourceModel
from moxn_types.blocks.signed import SignedURLContentModel
from moxn_types.blocks.text import TextContentModel
from moxn_types.blocks.variable import VariableContentModel

ContentBlockModel = (
    ImageContentFromSourceModel
    | PDFContentFromSourceModel
    | SignedURLContentModel
    | TextContentModel
    | VariableContentModel
)

ContentBlockAdapter: TypeAdapter[ContentBlockModel] = TypeAdapter(ContentBlockModel)



from typing import Sequence

from moxn_types.type_aliases.anthropic import AnthropicContentBlockParam
from moxn_types.type_aliases.google import GoogleContentBlock
from moxn_types.type_aliases.openai_chat import OpenAIChatContentBlock

ProviderContentBlock = (
    AnthropicContentBlockParam | OpenAIChatContentBlock | GoogleContentBlock
)


ProviderContentBlockSequence = (
    Sequence[Sequence[AnthropicContentBlockParam]]
    | Sequence[Sequence[OpenAIChatContentBlock]]
    | Sequence[Sequence[GoogleContentBlock]]
)



"""
Module for parsing and normalizing LLM responses across different providers.
"""

from enum import Enum
from typing import (
    Any,
    Generic,
    Protocol,
    TypeVar,
)
from uuid import UUID, uuid4

from pydantic import BaseModel, ConfigDict, Field

from moxn_types.blocks.text import TextContentModel
from moxn_types.blocks.tool import ToolCallModel
from moxn_types.content import Provider


class StopReason(str, Enum):
    """Normalized stop reasons across providers."""

    END_TURN = "end_turn"  # Normal completion
    MAX_TOKENS = "max_tokens"  # Max tokens reached
    TOOL_CALL = "tool_call"  # Model wants to call a tool
    CONTENT_FILTER = "content_filter"  # Content was filtered for safety reasons
    ERROR = "error"  # Some error occurred
    OTHER = "other"  # Other reason


class TokenUsage(BaseModel):
    """Normalized token usage across providers."""

    input_tokens: int | None = None
    thinking_tokens: int | None = None
    completion_tokens: int | None = None


class ResponseMetadata(BaseModel):
    """Normalized response metadata across providers."""

    normalized_finish_reason: StopReason
    raw_finish_reason: str


# Generic type parameters for text content and tool calls
TextContentT = TypeVar("TextContentT", bound=TextContentModel, covariant=True)
ToolCallT = TypeVar("ToolCallT", bound=ToolCallModel, covariant=True)


class ParsedResponseCandidateModelBase(BaseModel, Generic[TextContentT, ToolCallT]):
    """Normalized response candidate with content blocks and metadata."""

    content_blocks: list[TextContentT | ToolCallT]
    metadata: ResponseMetadata


class ParsedResponseCandidateModel(
    ParsedResponseCandidateModelBase[TextContentModel, ToolCallModel]
):
    """Normalized response candidate with content blocks and metadata."""

    content_blocks: list[TextContentModel | ToolCallModel]
    metadata: ResponseMetadata


ParsedResponseCandidateModelT = TypeVar(
    "ParsedResponseCandidateModelT", bound=ParsedResponseCandidateModelBase
)


class ParsedResponseModelBase(BaseModel, Generic[ParsedResponseCandidateModelT]):
    """
    Normalized response content from any LLM provider.

    Contains parsed content blocks, metadata, and original response for reference.
    """

    id: UUID = Field(default_factory=uuid4)
    provider: Provider
    candidates: list[ParsedResponseCandidateModelT]
    stop_reason: StopReason = Field(
        ...,
        description="Normalized stop reason - from first candidate if multiple candidates",
    )
    usage: TokenUsage = Field(
        default_factory=TokenUsage,
        description="Token usage from parent objects, candidate token usage in candidate metadata if available",
    )
    model: str | None = None
    raw_response: dict | None = None

    model_config = ConfigDict(arbitrary_types_allowed=True)


ResponseType = TypeVar("ResponseType", contravariant=True)
ParsedResponseModelT = TypeVar(
    "ParsedResponseModelT", bound=ParsedResponseModelBase, covariant=True
)


class ResponseParserProtocol(Protocol, Generic[ResponseType, ParsedResponseModelT]):
    """Protocol for provider-specific response parsers."""

    @classmethod
    def parse_response(
        cls, response: ResponseType, provider: Provider
    ) -> ParsedResponseModelT: ...

    @classmethod
    def extract_metadata(cls, response: ResponseType) -> dict[str, Any]: ...



from enum import Enum
from pathlib import Path
from typing import TYPE_CHECKING, Annotated, Any, ClassVar, Literal

from pydantic import Field

from moxn_types.blocks.base import BaseContent, BlockType

if TYPE_CHECKING:
    from google.genai.types import File as GoogleFile
else:
    GoogleFile = Any


class MediaDataImageFormat(str, Enum):
    BASE64 = "base64"
    BYTES = "bytes"
    GOOGLE_FILE = "google_file"
    GOOGLE_FILE_REFERENCE = "google_file_reference"
    LOCAL_FILE = "local_file"
    URL = "url"


class _ImageBase(BaseContent):
    """Base class for image content with common properties."""

    block_type: ClassVar[Literal[BlockType.IMAGE]] = BlockType.IMAGE

    media_type: Literal["image/jpeg", "image/png", "image/gif", "image/webp"]


class MediaImageFromBase64Model(_ImageBase):
    """
    Image encoded in base64 format.
    """

    type: Literal[MediaDataImageFormat.BASE64] = MediaDataImageFormat.BASE64
    base64: str


class MediaImageFromBytesModel(_ImageBase):
    """
    Image provided as raw bytes.
    """

    type: Literal[MediaDataImageFormat.BYTES] = MediaDataImageFormat.BYTES
    bytes: bytes


class MediaImageFromURLModel(_ImageBase):
    """
    Image referenced by a URL.
    Note: For Google Vertex AI, public HTTP URLs are NOT supported for images.
    """

    type: Literal[MediaDataImageFormat.URL] = MediaDataImageFormat.URL
    url: str


class MediaImageFromLocalFileModel(_ImageBase):
    """
    Image loaded from a local file path.
    """

    type: Literal[MediaDataImageFormat.LOCAL_FILE] = MediaDataImageFormat.LOCAL_FILE
    filepath: Path


class MediaImageFromGoogleFileModel(_ImageBase):
    """
    Image provided as a Google File object.
    Only supported by Google Gemini.
    """

    type: Literal[MediaDataImageFormat.GOOGLE_FILE] = MediaDataImageFormat.GOOGLE_FILE
    file: GoogleFile


class MediaImageFromGoogleFileReferenceModel(_ImageBase):
    """
    Image referenced by a Google File URI.
    For Gemini Developer API: can be any URI
    For Vertex AI: must be a GCS URI (gs://)
    """

    type: Literal[MediaDataImageFormat.GOOGLE_FILE_REFERENCE] = (
        MediaDataImageFormat.GOOGLE_FILE_REFERENCE
    )
    uri: str  # gs://


_ImageContentFromSourceTypes = (
    MediaImageFromBase64Model
    | MediaImageFromBytesModel
    | MediaImageFromURLModel
    | MediaImageFromLocalFileModel
    | MediaImageFromGoogleFileModel
    | MediaImageFromGoogleFileReferenceModel
)
ImageContentFromSourceModel = Annotated[
    _ImageContentFromSourceTypes,
    Field(discriminator="type"),
]



from datetime import datetime
from enum import Enum
from typing import Literal, overload
from uuid import UUID, uuid4

from pydantic import BaseModel, Field


class MessageRole(str, Enum):
    SYSTEM = "system"
    USER = "user"
    ASSISTANT = "assistant"
    SCHEMA = "schema"
    DEVELOPER = "developer"
    TOOL_CALL = "tool_call"
    TOOL_RESULT = "tool_result"
    MODEL = "model"


ANTHROPIC_MESSAGE_ROLES = Literal[
    MessageRole.SYSTEM,
    MessageRole.USER,
    MessageRole.ASSISTANT,
    MessageRole.TOOL_CALL,
    MessageRole.TOOL_RESULT,
]

OPENAI_MESSAGE_ROLES = Literal[
    MessageRole.SYSTEM,
    MessageRole.DEVELOPER,
    MessageRole.USER,
    MessageRole.ASSISTANT,
    MessageRole.TOOL_CALL,
    MessageRole.TOOL_RESULT,
]

GOOGLE_GEMINI_MESSAGE_ROLES = Literal[
    MessageRole.SYSTEM,
    MessageRole.USER,
    MessageRole.MODEL,
    MessageRole.TOOL_CALL,
    MessageRole.TOOL_RESULT,
]

GOOGLE_VERTEX_MESSAGE_ROLES = Literal[
    MessageRole.SYSTEM,
    MessageRole.USER,
    MessageRole.MODEL,
    MessageRole.TOOL_CALL,
    MessageRole.TOOL_RESULT,
]

ANTHROPIC_CONTENT_MESSAGE_ROLES = Literal[
    MessageRole.USER,
    MessageRole.ASSISTANT,
    MessageRole.TOOL_CALL,
    MessageRole.TOOL_RESULT,
]


GOOGLE_GEMINI_CONTENT_MESSAGE_ROLES = Literal[
    MessageRole.USER,
    MessageRole.MODEL,
    MessageRole.TOOL_CALL,
    MessageRole.TOOL_RESULT,
]


GOOGLE_VERTEX_CONTENT_MESSAGE_ROLES = Literal[
    MessageRole.USER,
    MessageRole.MODEL,
    MessageRole.TOOL_CALL,
    MessageRole.TOOL_RESULT,
]


class Provider(Enum):
    ANTHROPIC = "anthropic"
    OPENAI_CHAT = "openai_chat"
    GOOGLE_GEMINI = "google_gemini"
    GOOGLE_VERTEX = "google_vertex"


class Author(Enum):
    HUMAN = "human"
    MACHINE = "machine"


@overload
def map_role_to_provider(
    role: MessageRole, provider: Literal[Provider.ANTHROPIC]
) -> ANTHROPIC_MESSAGE_ROLES: ...


@overload
def map_role_to_provider(
    role: MessageRole, provider: Literal[Provider.OPENAI_CHAT]
) -> OPENAI_MESSAGE_ROLES: ...


@overload
def map_role_to_provider(
    role: MessageRole, provider: Literal[Provider.GOOGLE_GEMINI]
) -> GOOGLE_GEMINI_MESSAGE_ROLES: ...


@overload
def map_role_to_provider(
    role: MessageRole, provider: Literal[Provider.GOOGLE_VERTEX]
) -> GOOGLE_VERTEX_MESSAGE_ROLES: ...


def map_role_to_provider(role: MessageRole, provider: Provider) -> MessageRole:
    """
    Maps a MessageRole to the appropriate role for a specific provider.

    Args:
        role: The original MessageRole
        provider: The target provider

    Returns:
        The appropriate MessageRole for the specified provider
    """
    if provider == Provider.OPENAI_CHAT:
        if role == MessageRole.MODEL:
            return MessageRole.ASSISTANT
        return role
    elif provider == Provider.ANTHROPIC:
        if role == MessageRole.DEVELOPER:
            return MessageRole.SYSTEM
        elif role == MessageRole.MODEL:
            return MessageRole.ASSISTANT
        return role
    elif provider in (Provider.GOOGLE_GEMINI, Provider.GOOGLE_VERTEX):
        if role == MessageRole.ASSISTANT:
            return MessageRole.MODEL
        elif role == MessageRole.DEVELOPER:
            return MessageRole.SYSTEM
        return role
    else:
        raise ValueError(f"Unsupported provider: {provider}")


class SignedURLContentRequest(BaseModel):
    id: UUID = Field(default_factory=uuid4)
    file_path: str
    ttl_seconds: int = 3600


class SignedURLContentResponse(BaseModel):
    id: UUID = Field(default_factory=uuid4)
    signed_url: str
    expiration: datetime
    message: str = "Signed URL generated successfully"



from datetime import datetime
from typing import (
    Any,
    Generic,
    Literal,
    Protocol,
    Sequence,
    TypeVar,
    runtime_checkable,
)
from uuid import UUID, uuid4

from pydantic import BaseModel, ConfigDict, Field, SecretStr

from moxn_types.blocks.content_block import ContentBlockModel
from moxn_types.content import Author, MessageRole
from moxn_types.schema import MoxnMessageMetadata

T = TypeVar("T")


@runtime_checkable
class RenderableModel(Protocol):
    moxn_version_config: MoxnMessageMetadata

    def model_dump(
        self,
        *,
        mode: Literal["json", "python"] | str = "python",
        include: Any = None,
        exclude: Any = None,
        context: dict[str, Any] | None = None,
        by_alias: bool = False,
        exclude_unset: bool = False,
        exclude_defaults: bool = False,
        exclude_none: bool = False,
        round_trip: bool = False,
        warnings: bool | Literal["none", "warn", "error"] = True,
        serialize_as_any: bool = False,
    ) -> Any: ...

    def render(self, **kwargs: Any) -> Any: ...


class BaseHeaders(BaseModel):
    user_id: str
    org_id: str | None = None
    api_key: SecretStr

    def to_headers(self) -> dict[str, str]:
        return {
            "user_id": self.user_id,
            "org_id": self.org_id or "",
            "api_key": self.api_key.get_secret_value(),
        }


class MessageBase(BaseModel, Generic[T]):
    id: UUID = Field(default_factory=uuid4)
    version_id: UUID | None = Field(None, alias="versionId")
    name: str
    description: str
    author: Author
    role: MessageRole
    blocks: Sequence[Sequence[T]] = Field(repr=False)

    model_config = ConfigDict(populate_by_name=True)


class Message(MessageBase[ContentBlockModel]):
    blocks: Sequence[Sequence[ContentBlockModel]] = Field(repr=False)


class BasePrompt(BaseModel, Generic[T]):
    id: UUID
    version_id: UUID = Field(..., alias="versionId")
    user_id: UUID = Field(..., alias="userId")
    org_id: UUID | None = Field(None, alias="orgId")
    name: str
    description: str
    task_id: UUID = Field(..., alias="taskId")
    created_at: datetime = Field(..., alias="createdAt")
    messages: Sequence[T]
    message_order: Sequence[UUID] = Field(default_factory=list, alias="messageOrder")

    model_config = ConfigDict(populate_by_name=True)


class Prompt(BasePrompt[Message]):
    messages: Sequence[Message]


class BaseTask(BaseModel, Generic[T]):
    id: UUID
    version_id: UUID = Field(..., alias="versionId")
    user_id: UUID = Field(..., alias="userId")
    org_id: UUID | None = Field(None, alias="orgId")
    name: str
    description: str
    created_at: datetime = Field(..., alias="createdAt")
    prompts: Sequence[T]

    model_config = ConfigDict(populate_by_name=True)


class Task(BaseTask[Prompt]):
    prompts: Sequence[Prompt]



from typing import Any, ClassVar, Generic, Literal, TypeVar

from pydantic import BaseModel

from moxn_types.blocks.base import BlockType
from moxn_types.blocks.image import ImageContentFromSourceModel
from moxn_types.blocks.text import TextContentModel

T = TypeVar("T")


class ToolCallModel(BaseModel):
    block_type: ClassVar[Literal[BlockType.TOOL_CALL]] = BlockType.TOOL_CALL
    id: str
    arguments: str | dict[str, Any] | None
    name: str


class ToolResultBase(BaseModel, Generic[T]):
    block_type: ClassVar[Literal[BlockType.TOOL_RESULT]] = BlockType.TOOL_RESULT
    type: Literal["tool_use"]
    id: str
    name: str
    content: T | None


class ToolResultModel(
    ToolResultBase[TextContentModel | ImageContentFromSourceModel | None]
):
    type: Literal["tool_use"]
    id: str
    name: str
    content: TextContentModel | ImageContentFromSourceModel | None



from enum import Enum
from typing import Annotated, ClassVar, Literal

from pydantic import Field

from moxn_types.blocks.base import BaseContent, BlockType


# 1. Enums
# --------------------------------------------------------------------------- #
class VariableFormat(str, Enum):
    INLINE = "inline"
    BLOCK = "block"


class VariableType(str, Enum):
    """Format of a variable."""

    PRIMITIVE = "primitive"
    IMAGE = "image"
    DOCUMENT = "document"


# --------------------------------------------------------------------------- #
# 2. Base Variable class
# --------------------------------------------------------------------------- #


class TextVariableModel(BaseContent):
    """A variable that represents text content."""

    block_type: ClassVar[Literal[BlockType.VARIABLE]] = BlockType.VARIABLE
    name: str
    variable_type: Literal[VariableType.PRIMITIVE] = VariableType.PRIMITIVE
    format: VariableFormat
    description: str = ""
    required: bool = True
    default_value: str | None = None


class ImageVariableModel(BaseContent):
    """A variable that represents image content."""

    block_type: ClassVar[Literal[BlockType.VARIABLE]] = BlockType.VARIABLE
    name: str
    variable_type: Literal[VariableType.IMAGE] = VariableType.IMAGE
    format: VariableFormat
    description: str = ""
    required: bool = True


class DocumentVariableModel(BaseContent):
    """A variable that represents document content (PDF)."""

    block_type: ClassVar[Literal[BlockType.VARIABLE]] = BlockType.VARIABLE
    name: str
    variable_type: Literal[VariableType.DOCUMENT] = VariableType.DOCUMENT
    format: VariableFormat
    description: str = ""
    required: bool = True


VariableContentModel = Annotated[
    TextVariableModel | ImageVariableModel | DocumentVariableModel,
    Field(discriminator="variable_type"),
]



from typing import ClassVar, Literal

from moxn_types.blocks.base import BaseContent, BlockType


class TextContentModel(BaseContent):
    block_type: ClassVar[Literal[BlockType.TEXT]] = BlockType.TEXT
    text: str



from enum import Enum
from typing import Any

from pydantic import BaseModel


class BlockType(Enum):
    DOCUMENT = "document"
    IMAGE = "image"
    TEXT = "text"
    VARIABLE = "variable"
    SIGNED = "signed"
    TOOL_CALL = "tool_call"
    TOOL_RESULT = "tool_result"


class BaseContent(BaseModel):
    options: dict[str, Any] = {}



"""Type definitions for Google provider-specific content blocks."""

from typing import TYPE_CHECKING, Sequence, TypedDict, Union

from pydantic import BaseModel

if TYPE_CHECKING:
    # -- Google --
    from google.genai.types import (
        Candidate as GoogleGenerateContentResponseCandidate,
    )
    from google.genai.types import Content as GoogleContent
    from google.genai.types import File as GoogleFile
    from google.genai.types import FunctionCall as GoogleFunctionCall
    from google.genai.types import FunctionResponse as GoogleFunctionResponse
    from google.genai.types import (
        GenerateContentResponse as GoogleGenerateContentResponse,
    )
    from google.genai.types import Part as GooglePart
else:
    GoogleContent = dict
    GoogleFile = dict
    GoogleFunctionCall = dict
    GoogleFunctionResponse = dict
    GooglePart = dict
    GoogleGenerateContentResponse = BaseModel
try:
    from google.genai.types import (
        Candidate as GoogleGenerateContentResponseCandidate,
    )
    from google.genai.types import Content as GoogleContent
    from google.genai.types import File as GoogleFile
    from google.genai.types import FunctionCall as GoogleFunctionCall
    from google.genai.types import FunctionResponse as GoogleFunctionResponse
    from google.genai.types import (
        GenerateContentResponse as GoogleGenerateContentResponse,
    )
    from google.genai.types import Part as GooglePart
except ImportError:
    pass


# Google content block types
GoogleContentBlock = Union[
    GooglePart,
    GoogleFile,
    GoogleFunctionCall,
    GoogleFunctionResponse,
]

GoogleSystemContentBlock = Union[
    GooglePart,
    GoogleFile,
]

# Provider-specific block sequences (for grouping operations)
GoogleContentBlockSequence = Sequence[Sequence[GoogleContentBlock]]
GoogleSystemContentBlockSequence = Sequence[Sequence[GoogleSystemContentBlock]]


class GoogleMessagesParam(TypedDict, total=False):
    system_instruction: str
    content: Sequence[GoogleContent | GoogleFile]



from datetime import datetime
from enum import Enum
from typing import Any, Generic, Literal, Optional, Protocol, TypeAlias, TypeVar, Union
from uuid import UUID, uuid4

from pydantic import BaseModel, ConfigDict, Field

from moxn_types.content import Provider
from moxn_types.core import Message, MessageBase, RenderableModel
from moxn_types.response import ParsedResponseModelBase


# Core Domain Types
class SpanKind(str, Enum):
    LLM = "llm"
    TOOL = "tool"
    AGENT = "agent"


class SpanStatus(str, Enum):
    UNSET = "UNSET"
    OK = "OK"
    ERROR = "ERROR"


class SpanLogType(str, Enum):
    START = "span_start"
    END = "span_end"
    ERROR = "span_error"


class SpanEventLogType(str, Enum):
    EVENT = "span_event"
    ERROR = "span_event_error"


class BaseTelemetryLogRequest(BaseModel):
    id: UUID
    timestamp: datetime | None = None
    prompt_id: UUID
    prompt_version_id: UUID
    message: str | None = None
    log_metadata: dict[str, Any] = Field(default_factory=dict)
    attributes: dict[str, Any] = Field(default_factory=dict)
    attributes_key: str | None = None


class SpanLogRequest(BaseTelemetryLogRequest):
    span_id: UUID
    root_span_id: UUID
    parent_span_id: UUID | None = None
    event_type: SpanLogType


class SpanEventLogRequest(BaseTelemetryLogRequest):
    span_id: UUID
    span_event_id: UUID
    event_type: SpanEventLogType


# Base Models
class BaseTelemetryEvent(BaseModel):
    """Base class for all telemetry events"""

    model_config = ConfigDict(
        json_encoders={
            UUID: str,  # Ensure UUIDs are serialized as strings
            datetime: lambda dt: dt.isoformat(),  # Ensure datetimes are ISO format
        }
    )

    id: UUID
    timestamp: datetime
    prompt_id: UUID
    prompt_version_id: UUID
    message: Optional[str] = None
    attributes: dict[str, Any] = Field(default_factory=dict)


class BaseSpanLog(BaseTelemetryEvent):
    """Base class for span logs"""

    span_id: UUID
    root_span_id: UUID
    parent_span_id: Optional[UUID] = None
    event_type: SpanLogType


class BaseSpanEventLog(BaseTelemetryEvent):
    """Base class for span event logs"""

    span_id: UUID
    span_event_id: UUID
    event_type: SpanEventLogType


class TelemetryLogResponse(BaseModel):
    """Response from telemetry log endpoint"""

    id: UUID
    timestamp: datetime
    status: str = "success"
    message: Optional[str] = None


class BaseSpan(BaseTelemetryEvent):
    """Base class for span-related events"""

    span_id: UUID
    name: str
    kind: SpanKind
    status: SpanStatus = SpanStatus.UNSET
    root_span_id: Optional[UUID] = None
    parent_span_id: Optional[UUID] = None


class BaseSpanEvent(BaseTelemetryEvent):
    """Base class for span event-related events"""

    span_id: UUID
    event_type: Literal["llm_response"]
    variables: Optional[dict[str, Any]] = None
    messages: Optional[list[dict[str, Any]]] = None
    llm_response_content: Optional[str] = None
    llm_response_tool_calls: Optional[list[dict[str, Any]]] = None


# Domain Events
class SpanCreated(BaseSpan):
    """Event emitted when a span is created"""

    pass


class SpanCompleted(BaseSpan):
    """Event emitted when a span is completed"""

    pass


class SpanFailed(BaseSpan):
    """Event emitted when a span fails"""

    error: str


class SpanResponse(BaseModel):
    span_id: UUID
    status: str = "success"
    message: Optional[str] = None


class SpanEventResponse(BaseModel):
    event_id: UUID
    span_id: UUID
    event_type: str
    status: str = "success"
    message: Optional[str] = None


class LLMSpanEvent(BaseSpanEvent):
    """Event emitted for LLM interactions"""

    provider: Provider
    raw_input: Optional[dict[str, Any]] = None
    rendered_input: Optional[dict[str, Any]] = None


class CreateSpanRequest(BaseSpan):
    """API prompt model for span creation"""

    pass


class CreateSpanEventRequest(BaseSpanEvent):
    """API prompt model for span event creation"""

    pass


class TelemetryResponse(BaseTelemetryEvent):
    """Base API response model"""

    status: str = "success"


class ErrorResponse(BaseTelemetryEvent):
    """API error response model"""

    status: str = "error"
    error_message: str


# --- Type Aliases ---
TelemetryLogRequest: TypeAlias = Union[SpanLogRequest, SpanEventLogRequest]


class Entity(BaseModel):
    entity_type: str
    entity_id: UUID
    entity_version_id: UUID | None = None


class SignedURLRequest(BaseModel):
    id: UUID = Field(default_factory=uuid4)
    file_path: str
    entity: Entity | None = None
    log_request: TelemetryLogRequest
    media_type: Literal[
        "image/jpeg",
        "image/png",
        "image/gif",
        "image/webp",
        "application/pdf",
        "application/json",
    ]
    prompt_id: UUID | None = None
    prompt_version_id: UUID | None = None


class SignedURLResponse(BaseModel):
    id: UUID = Field(default_factory=uuid4)
    url: str
    file_path: str
    expiration: datetime
    message: str = "Signed URL generated successfully"


MAX_INLINE_ATTRIBUTES_SIZE = 1  # 10 * 1024  # 10KB threshold for inline attributes


# Transport Protocol
class TelemetryTransport(Protocol):
    """Protocol for sending telemetry data"""

    async def send_log(
        self, log_request: Union[SpanLogRequest, SpanEventLogRequest]
    ) -> TelemetryLogResponse: ...

    async def send_telemetry_log_and_get_signed_url(
        self, log_request: SignedURLRequest
    ) -> SignedURLResponse: ...


ParsedResponseT = TypeVar("ParsedResponseT", bound=ParsedResponseModelBase)
MessageT = TypeVar("MessageT", bound=MessageBase)


class LLMEventModelBase(BaseModel, Generic[ParsedResponseT, MessageT]):
    """Domain model for LLM interactions"""

    messages: list[MessageT]
    provider: Provider | None = None
    raw_response: dict[str, Any]
    parsed_response: ParsedResponseT
    session_data: RenderableModel | None = None
    rendered_input: Optional[dict[str, Any]] = None
    attributes: Optional[dict[str, Any]] = None

    model_config = ConfigDict(
        arbitrary_types_allowed=True,
    )


class LLMEventModel(LLMEventModelBase[ParsedResponseModelBase, Message]):
    """Domain model for LLM interactions"""

    messages: list[Message]
    parsed_response: ParsedResponseModelBase



from enum import Enum
from typing import Any, Literal, Optional

from pydantic import BaseModel, ConfigDict, Field

from moxn_types.base import NOT_GIVEN, BaseModelWithOptionalFields, NotGivenOr


# Define Schema types as proper enums to match OpenAPI compatibility
class SchemaPropertyType(str, Enum):
    STRING = "string"
    INTEGER = "integer"
    NUMBER = "number"
    BOOLEAN = "boolean"
    OBJECT = "object"
    ARRAY = "array"
    REFERENCE = "reference"  # Custom extension for references


class SchemaPropertyFormat(str, Enum):
    TEXT = "text"
    DATE = "date"
    DATETIME = "date-time"
    EMAIL = "email"
    URI = "uri"
    UUID = "uuid"
    HOSTNAME = "hostname"
    IPV4 = "ipv4"
    IPV6 = "ipv6"
    BINARY = "binary"


# Updated to match OpenAPI spec and TypeScript interface
class SchemaPropertyConstraints(BaseModel):
    # Common
    nullable: bool | None = None
    enum: list[Any] | None = None
    default: Any | None = None

    # String constraints
    minLength: int | None = None
    maxLength: int | None = None
    pattern: str | None = None
    format: SchemaPropertyFormat | None = None

    # Number constraints
    minimum: int | float | None = None
    maximum: int | float | None = None
    exclusiveMinimum: bool | None = None
    exclusiveMaximum: bool | None = None
    multipleOf: int | float | None = None

    # Array constraints
    minItems: int | None = None
    maxItems: int | None = None
    uniqueItems: bool | None = None

    # Object constraints
    additionalProperties: bool | None = None
    required: list[str] | None = None

    # Variable Display
    displayMode: str | None = None  # 'inline' | 'block'


class SchemaReference(BaseModel):
    messageId: str
    schemaName: str
    reference: str
    ref: str | None = Field(None, alias="$ref")  # OpenAPI style reference
    isSelfReference: bool | None = None


class SchemaProperty(BaseModelWithOptionalFields):
    name: str
    type: SchemaPropertyType
    description: str
    required: NotGivenOr[bool] = NOT_GIVEN
    properties: NotGivenOr[list["SchemaProperty"]] = NOT_GIVEN  # For object type
    items: NotGivenOr[Optional["SchemaProperty"]] = (
        NOT_GIVEN  # For array type (singular)
    )
    constraints: NotGivenOr[SchemaPropertyConstraints] = NOT_GIVEN
    schemaRef: NotGivenOr[SchemaReference] = (
        NOT_GIVEN  # Updated to use SchemaReference class
    )
    defaultContent: NotGivenOr[Any] = NOT_GIVEN


class Schema(BaseModel):
    name: str
    description: str | None = None
    type: Literal["object"] | None = None  # Default type for top-level schemas
    properties: list[SchemaProperty]
    required: list[str] | None = None  # OpenAPI required property names
    strict: bool = False


class SchemaPromptType(str, Enum):
    ALL = "all"
    INPUT = "input"
    OUTPUT = "output"


class MessageType(str, Enum):
    PROMPT = "message"
    SCHEMA_INPUT = "input"
    SCHEMA_OUTPUT = "output"


class SchemaWithMetadata(BaseModel):
    moxn_schema: Schema = Field(alias="schema")
    created_at: str = Field(alias="createdAt")
    updated_at: str = Field(alias="updatedAt")
    message_id: str = Field(alias="messageId")
    message_version_id: str = Field(alias="messageVersionId")
    prompt_id: str = Field(alias="promptId")
    prompt_version_id: str = Field(alias="promptVersionId")
    message_type: MessageType = Field(alias="schemaType")

    model_config = ConfigDict(
        populate_by_name=True,  # Allow both alias and Python names
        alias_generator=lambda s: "".join(
            word.capitalize() if i > 0 else word for i, word in enumerate(s.split("_"))
        ),
    )


class PromptSchemas(BaseModel):
    input: SchemaWithMetadata | None = None
    outputs: list[SchemaWithMetadata] = Field(default_factory=list)

    model_config = ConfigDict(populate_by_name=True)


class MoxnMessageMetadata(BaseModel):
    message_id: str
    message_version_id: str
    prompt_id: str
    prompt_version_id: str


class MoxnBaseModel(BaseModel):
    """Base model that includes message metadata configuration"""

    model_version_config: dict[str, MoxnMessageMetadata | None] = {
        "metadata": None  # Initially None, must be set before model instantiation
    }

    @classmethod
    def set_metadata(cls, metadata: MoxnMessageMetadata) -> None:
        """Set the metadata for this model class"""
        cls.model_version_config["metadata"] = metadata


class CodegenResponse(BaseModel):
    """Response model for code generation prompts"""

    files: dict[str, str]


SchemaProperty.model_rebuild()



"""Type definitions for OpenAI Chat provider-specific content blocks."""

from typing import TYPE_CHECKING, Sequence, TypedDict, Union

if TYPE_CHECKING:
    # -- OpenAIChat --
    from openai.types.chat import ChatCompletion as OpenAIChatCompletion
    from openai.types.chat import (
        ChatCompletionMessageToolCallParam as OpenAIChatToolUseBlockParam,
    )
    from openai.types.chat.chat_completion_assistant_message_param import (
        ChatCompletionAssistantMessageParam as OpenAIChatAssistantMessageParam,
    )
    from openai.types.chat.chat_completion_content_part_image_param import (
        ChatCompletionContentPartImageParam as OpenAIChatCompletionContentPartImageParam,
    )
    from openai.types.chat.chat_completion_content_part_image_param import (
        ImageURL as OpenAIChatImageURL,
    )
    from openai.types.chat.chat_completion_content_part_param import (
        ChatCompletionContentPartParam as OpenAIChatCompletionContentPartParam,
    )
    from openai.types.chat.chat_completion_content_part_param import (
        File as OpenAIChatFile,
    )
    from openai.types.chat.chat_completion_content_part_param import (
        FileFile as OpenAIChatFileFile,
    )
    from openai.types.chat.chat_completion_content_part_text_param import (
        ChatCompletionContentPartTextParam as OpenAIChatCompletionContentPartTextParam,
    )
    from openai.types.chat.chat_completion_developer_message_param import (
        ChatCompletionDeveloperMessageParam as OpenAIChatDeveloperMessageParam,
    )
    from openai.types.chat.chat_completion_message_tool_call_param import (
        Function as OpenAIChatFunction,
    )
    from openai.types.chat.chat_completion_system_message_param import (
        ChatCompletionSystemMessageParam as OpenAIChatSystemMessageParam,
    )
    from openai.types.chat.chat_completion_tool_message_param import (
        ChatCompletionToolMessageParam as OpenAIChatToolResponseParam,
    )
    from openai.types.chat.chat_completion_user_message_param import (
        ChatCompletionUserMessageParam as OpenAIChatUserMessageParam,
    )
else:
    OpenAIChatToolUseBlockParam = dict
    OpenAIChatToolResponseParam = dict
    OpenAIChatCompletionContentPartParam = dict
    OpenAIChatCompletionContentPartImageParam = dict
    OpenAIChatImageURL = dict
    OpenAIChatFile = dict
    OpenAIChatFileFile = dict
    OpenAIChatToolUseBlockParam = dict
    OpenAIChatToolResponseParam = dict
    OpenAIChatCompletionContentPartTextParam = dict
    OpenAIChatFunction = dict
    OpenAIChatAssistantMessageParam = dict
    OpenAIChatDeveloperMessageParam = dict
    OpenAIChatSystemMessageParam = dict
    OpenAIChatUserMessageParam = dict
    try:
        from openai.types.chat import ChatCompletion as OpenAIChatCompletion
    except ImportError:
        pass

# OpenAI content block types
OpenAIChatContentBlock = Union[
    OpenAIChatCompletionContentPartParam,
    OpenAIChatCompletionContentPartImageParam,
    OpenAIChatToolUseBlockParam,
    OpenAIChatToolResponseParam,
]

# Provider-specific block sequences (for grouping operations)
OpenAIChatContentBlockSequence = Sequence[Sequence[OpenAIChatContentBlock]]


class OpenAIChatMessagesParam(TypedDict):
    messages: list[OpenAIChatContentBlock]
